// 
// Copyright (C) 2016, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.ComponentModel;
using NinjaTrader;
using NinjaTrader.Cbi;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
#endregion

namespace NinjaTrader.NinjaScript.BarsTypes
{
	public class WickedRenkoNT : BarsType
	{
		private			double			offset;
		private			double			renkoHigh;
		private			double			renkoLow;
		private  		int				numberSame;
		private 		bool 			trend = false;

		public override void ApplyDefaultBasePeriodValue(BarsPeriod period)
		{
			
		}

		public override void ApplyDefaultValue(BarsPeriod period)
		{
			period.BarsPeriodTypeName = "WickedRenko";
			period.Value = 2;
		}

		public override string ChartLabel(DateTime time)
		{
			return time.ToString("T", Core.Globals.GeneralOptions.CurrentCulture);
		}

		public override int GetInitialLookBackDays(BarsPeriod period, TradingHours tradingHours, int barsBack)
		{
			return 3;
		}

		public override double GetPercentComplete(Bars bars, DateTime now)
		{
			return 0;
		}

		public override bool IsRemoveLastBarSupported { get { return false; } }

		protected override void OnDataPoint(Bars bars, double open, double high, double low, double close, DateTime time, long volume, bool isBar, double bid, double ask)
		{
			if (SessionIterator == null)
				SessionIterator = new SessionIterator(bars);

			offset = bars.BarsPeriod.Value * bars.Instrument.MasterInstrument.TickSize;
			bool isNewSession = SessionIterator.IsNewSession(time, isBar);
			if (isNewSession)
				SessionIterator.GetNextSession(time, isBar);

			if (bars.Count == 0 || bars.IsResetOnNewTradingDay && isNewSession)
			{
                double op = Math.Round(close / offset) * offset;
				renkoHigh	= op + offset;
				renkoLow	= op - offset;
				double hi, lo;
				if(op > close) {
					hi = op;
					lo = close;
				}else{
					hi = close;
					lo = op;
				}
                isNewSession = SessionIterator.IsNewSession(time, isBar);
				if (isNewSession)
					SessionIterator.GetNextSession(time, isBar);

				AddBar(bars, op, hi, lo, close, time, volume);
				bars.LastPrice = close;				
				return;
			}

			double		barOpen		= bars.GetOpen(bars.Count - 1);
			double		barHigh		= bars.GetHigh(bars.Count - 1);
			double		barLow		= bars.GetLow(bars.Count - 1);
			long		barVolume	= bars.GetVolume(bars.Count - 1);
			DateTime	barTime		= bars.GetTime(bars.Count - 1);

			if (renkoHigh == 0 || renkoLow == 0)
			{
				if (bars.Count == 1)
				{
					renkoHigh	= barOpen + offset;
					renkoLow	= barOpen - offset;
				}
				else if (bars.GetClose(bars.Count - 2) > bars.GetOpen(bars.Count - 2))
				{
					renkoHigh	= bars.GetClose(bars.Count - 2) + offset;
					renkoLow	= bars.GetClose(bars.Count - 2) - offset * 2;
				}
				else
				{
					renkoHigh	= bars.GetClose(bars.Count - 2) + offset * 2;
					renkoLow	= bars.GetClose(bars.Count - 2) - offset;
				}
			}
			
			if (close.ApproxCompare(renkoHigh) >= 0)
			{
				double lastBarClose = bars.GetClose(bars.Count - 1);
				UpdateBar(bars, lastBarClose, lastBarClose, bars.GetOpen(bars.Count - 1)+ (trend ? offset:2*offset), bars.GetTime(bars.Count - 1), 0);	

				renkoLow	= renkoHigh - 2.0 * offset;
				renkoHigh	= renkoHigh + offset;

				isNewSession = SessionIterator.IsNewSession(time, isBar);
				if (isNewSession)
					SessionIterator.GetNextSession(time, isBar);

				while (close.ApproxCompare(renkoHigh) >= 0)	//Don't Add empty bars to fill gap if price jumps
				{
					renkoLow	= renkoHigh - 2.0 * offset;
					renkoHigh	= renkoHigh + offset;
				}

				// Add final partial bar
				AddBar(bars, renkoHigh - offset, Math.Max(renkoHigh - offset, close), Math.Min(renkoHigh - offset, close), close, time, volume);				
				trend = true;
			}
			else
				if (close.ApproxCompare(renkoLow) <= 0)
				
				{
					double lastBarClose = bars.GetClose(bars.Count - 1);
					UpdateBar(bars, lastBarClose, lastBarClose, bars.GetOpen(bars.Count - 1) - (trend ? 2*offset:offset), bars.GetTime(bars.Count - 1), 0);					

					renkoHigh	= renkoLow + 2.0 * offset;
					renkoLow	= renkoLow - offset;

					isNewSession = SessionIterator.IsNewSession(time, isBar);
					if (isNewSession)
						SessionIterator.GetNextSession(time, isBar);

					while (close.ApproxCompare(renkoLow) <= 0)	//Don't Add empty bars to fill gap if price jumps
					{
						renkoHigh	= renkoLow + 2.0 * offset;
						renkoLow	= renkoLow - offset;
					}

					// Add final partial bar
					AddBar(bars, renkoLow + offset, Math.Max(renkoLow + offset, close), Math.Min(renkoLow + offset, close), close, time, volume);
					trend = false;
				}
				else
					UpdateBar(bars, close, close, close, time, volume);					
			
				bars.LastPrice	= close;
		}

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"NT8 WickedRenko by PureAlgo";
				Name						= "WickedRenkoNT";
				
				BarsPeriod					= new BarsPeriod { BarsPeriodType = (BarsPeriodType) 125, BarsPeriodTypeName = "WickedRenkoNT(25)", Value = 1 };
				BuiltFrom					= BarsPeriodType.Tick;
				DaysToLoad					= 3;
				DefaultChartStyle			= Gui.Chart.ChartStyleType.CandleStick;
				IsIntraday					= true;
				IsTimeBased					= false;
			}
			else if (State == State.Configure)
			{
				Name = "WickedRenkoNT" + BarsPeriod.Value;
				Properties.Remove(Properties.Find("BaseBarsPeriodType",			true));
				Properties.Remove(Properties.Find("BaseBarsPeriodValue",		true));
				Properties.Remove(Properties.Find("PointAndFigurePriceType",	true));
				Properties.Remove(Properties.Find("ReversalType",				true));
				Properties.Remove(Properties.Find("Value2",						true));

				SetPropertyName("Value", Custom.Resource.NinjaScriptBarsTypeRenkoBrickSize);
			}
		}
	}
}
